% main_making_of_a_crisis.m
%
% Computes the results for the making-of-a-crisis scenario in Section 5 
% of the paper
% 'Household Debt and Crises of Confidence', Quantitative Economics,
%
% Thomas Hintermaier, thomas.hintermaier@uni-bonn.de and
% Winfried Koeniger,  winfried.koeniger@unisg.ch
%
% March 16, 2018
%
% Note: All relevant program components are provided 'as is',
% in particular, without any warranty whatsoever.
% Please cite the above-mentioned paper
% if you use this program or components thereof.
% ======================================================

clearvars;

plot_settings_collateral;

% Algorithm parameters
% ====================
max_card = 4;         % maximum number of equilibria per state
numb_muc_steps = 200; % number of steps between minimum and maximum marginal utility in line with equilibrium existence

% Model parameters
% ================
period_years = 10;

% the interest rate applicable during the final stage, i.e., to assets a_t, t >= T+1
r     = 1.04^period_years - 1; % interest rate on savings, BENCHMARK

beta_ = 1/(1 + r);

r_T   = 1.01^period_years - 1;    % interest rate on a_T
r_T_1 = 1.01^period_years - 1;  % interest rate on a_T_1
r_T_2 = 1.01^period_years - 1;  % interest rate on a_T_2

sigma_  = 10.0;     % overall risk aversion, BENCHMARK

theta = 0.70; %     % Cobb-Douglas weight on non-durable consumption

miu = 0.80; % loan-to-value ratio, BENCHMARK

y_T   = 1.03^period_years*period_years; % the endowment for the entire final stage, per period
y_T_1 = 1.0*period_years;
y_T_2 = 1.0*period_years;

% Create the grid on the state space
% ==================================

% h is housing
h_T   = 1.0;
h_T_1 = 1.0;
h_T_2 = 1.0;

% a_min is the lowest gridpoint at which c_T > 0
% despite ensuring positive consumption in period T,
% the endowment pattern in earlier periods might require
% lower values for the bound in preceeding periods
% USER: adjust grid in T - 1 and T - 2 if objects obtained display gaps
% (because of a grid specification which would discard some of the equilibria)

a_min = -y_T*(1 + r)/(r*(1 + r_T)) + eps;
a_max = 20*y_T;

a_small = 0.000000001;
numb_a_minus_gridpoints = 5500;
numb_a_plus_gridpoints  = 1500;

a_grid_minus = linspace(a_min  ,-a_small,numb_a_minus_gridpoints );
a_grid_plus  = linspace(a_small,   a_max,numb_a_plus_gridpoints  );

a_grid_ = [a_grid_minus,a_grid_plus]';  % set up exogenous grid over continuation-state (next-period state)
numb_a_gridpoints = length(a_grid_);

% =============================================================================
% Initialize equilibrium policy function and price function for terminal stage
% =============================================================================
c_pol = NaN*zeros(numb_a_gridpoints,1);
p_equ = NaN*zeros(numb_a_gridpoints,1);

c_pol(:,1) = r*(1 + r_T)/(1 + r)*a_grid_ + y_T;
p_equ(:,1) = (1 - theta)/theta*c_pol(:,1)/r;

% ==============================================================
% Intermediate period, T - 1
% ==============================================================

MUc = NaN*zeros(numb_a_gridpoints,1);
MUh = NaN*zeros(numb_a_gridpoints,1);

for iap = 1:numb_a_gridpoints
    if c_pol(iap,1) > 0
        % calculate partial derivatives of the utility function
        MUc(iap,1) =     theta  * (c_pol(iap,1)^(theta*(1 -sigma_)-1)) * h_T^((1-theta)*(1-sigma_));
        MUh(iap,1) =  (1-theta) *  c_pol(iap,1)^(theta*(1 -sigma_))    * h_T^((1-theta)*(1-sigma_) -1);
    end
end

unconstr_numer = (MUc.*p_equ + MUh)*1.0'*beta_;
unconstr_denom =  MUc*(1+r_T)      *1.0'*beta_;
% evaluating FOC for the unconstrained case
unconstr_FOC = unconstr_numer./unconstr_denom;

% initialize endogenous coordinates
a_EGM =        NaN*zeros(numb_a_gridpoints,1);
c_EGM =        NaN*zeros(numb_a_gridpoints,1);
p_EGM =        NaN*zeros(numb_a_gridpoints,1);
a_constr_EGM = NaN*zeros(numb_a_gridpoints,1);
c_constr_EGM = NaN*zeros(numb_a_gridpoints,1);
p_constr_EGM = NaN*zeros(numb_a_gridpoints,1);
kappa =        NaN*zeros(numb_a_gridpoints,1);

for iap = 1:numb_a_gridpoints
    
    if unconstr_FOC(iap,1) > -a_grid_(iap)*(1 + r_T)/(miu*h_T)
        % equilibrium for preceding states with non-binding constraints
        % -------------------------------------------------------------
        p_EGM(iap,1) = unconstr_FOC(iap,1);
        c_EGM(iap,1) = (unconstr_denom(iap,1)/(theta* h_T_1^((1-theta)*(1-sigma_)))).^(1/(theta*(1 - sigma_) - 1));
        a_EGM(iap,1) = (c_EGM(iap,1) + a_grid_(iap) - y_T_1)/(1 + r_T_1);
        
    end % of if on existence of unconstrained case, mapping into a_prime
    
    if a_grid_(iap) < 0 && unconstr_FOC(iap,1) >= -a_grid_(iap)*(1 + r_T)/(miu*h_T)
        % equilibrium for preceding states with binding constraints
        % ---------------------------------------------------------
        p_constr_EGM(iap,1) = -a_grid_(iap)*(1 + r_T)/(miu*h_T);
        
        kappa(iap,1) = 1/(1.0 + r_T - miu)*(unconstr_numer(iap,1)./p_constr_EGM(iap,1) - unconstr_denom(iap,1));
        
        c_constr_EGM(iap,1) = ( (unconstr_denom(iap,1) + kappa(iap,1)*(1 + r_T))/(theta*h_T_1^((1-theta)*(1-sigma_)))).^(1/(theta*(1 - sigma_) - 1));
        a_constr_EGM(iap,1) = 1.0/(1.0 + r_T_1)*(c_constr_EGM(iap,1) + a_grid_(iap) - y_T_1);
        
    end % of if on existence of constrained case (also requires negative financial assets)
    
end

% detect vertices and use them to locate kinks in policies
p_xp0 = -(1 + r_T)*a_grid_/(miu*h_T);
ap_cutoff = NaN*zeros(1,1);
a_cutoff_EGM = NaN*zeros(1,1);
c_cutoff_EGM = NaN*zeros(1,1);
p_cutoff_EGM = NaN*zeros(1,1);

sel1_not_NaN = ~isnan(unconstr_FOC(:,1));
ap_cutoff(1) = interp1(unconstr_FOC(sel1_not_NaN,1) - p_xp0(sel1_not_NaN),a_grid_(sel1_not_NaN),0);
if ap_cutoff(1) < 0
    p_cutoff_EGM(1) = -(1 + r_T)*ap_cutoff(1)/(miu*h_T);
    c_cutoff_EGM(1) = (1/(theta*h_T_1^((1-theta)*(1-sigma_)))*interp1(a_grid_(sel1_not_NaN),unconstr_denom(sel1_not_NaN,1),ap_cutoff(1)) ).^(1/(theta*(1 - sigma_) - 1));
    a_cutoff_EGM(1) = 1.0/(1.0 + r_T_1)*(c_cutoff_EGM(1) + ap_cutoff(1) - y_T_1);
end

% % figure(446);
% % plot(a_constr_EGM,a_grid_,'r','LineWidth',3);
% % hold on;
% % plot(a_EGM,a_grid_,       'g','LineWidth',3);
% % plot(a_cutoff_EGM,ap_cutoff,'.','Linewidth',3);
% % hold off;
% % xlabel('A_{T-1}')
% % ylabel('successor-state, A_{T}')
% % xlim([-6.0,1.0]);
% % nice_plot_medium2;
% % 
% % figure(447);
% % plot(a_constr_EGM,c_constr_EGM/period_years,'r','LineWidth',3);
% % hold on;
% % plot(a_EGM,c_EGM/period_years,              'g','LineWidth',3);
% % plot(a_cutoff_EGM,c_cutoff_EGM/period_years,'.','Linewidth',3);
% % hold off;
% % xlabel('A_{T-1}')
% % ylabel('cons. (annual), c_{T-1}')
% % xlim([-6.0,1.0]);
% % nice_plot_medium2;
% % 
% % figure(448);
% % plot(a_constr_EGM,p_constr_EGM,'r','LineWidth',3);
% % hold on;
% % plot(a_EGM,p_EGM,              'g','LineWidth',3);
% % plot(a_cutoff_EGM,p_cutoff_EGM,'.','LineWidth',3);
% % hold off;
% % xlabel('financial state, A_{T-1}')
% % ylabel('house price, p_{T-1}')
% % xlim([-6.0,1.0]);
% % nice_plot_medium2;

% splicing the relevant equilibrium curves, ensuring connectedness
sel_none_NaN_constr = ~isnan(a_constr_EGM) & ~isnan(c_constr_EGM) & ~isnan(p_constr_EGM);
sel_none_NaN        = ~isnan(a_EGM)        & ~isnan(c_EGM)        & ~isnan(p_EGM);

a_connect = cell(1,1);
c_connect = cell(1,1);
p_connect = cell(1,1);

sort_a_c_p_constr = sortrows([a_constr_EGM(sel_none_NaN_constr(:,1),1),c_constr_EGM(sel_none_NaN_constr(:,1),1),...
    p_constr_EGM(sel_none_NaN_constr(:,1),1),     a_grid_(sel_none_NaN_constr(:,1))],-4);

a_connect{1,1} = [sort_a_c_p_constr(:,1);a_cutoff_EGM(1);a_EGM(sel_none_NaN(:,1),1)];
c_connect{1,1} = [sort_a_c_p_constr(:,2);c_cutoff_EGM(1);c_EGM(sel_none_NaN(:,1),1)];
p_connect{1,1} = [sort_a_c_p_constr(:,3);p_cutoff_EGM(1);p_EGM(sel_none_NaN(:,1),1)];

clear sort_a_c_p_constr

% getting (potentially multiple) equilibrium policies and prices on state-grid
c_pol_1 = NaN*zeros(numb_a_gridpoints,1,max_card);
p_equ_1 = NaN*zeros(numb_a_gridpoints,1,max_card);

c_pol_1(:,1,:)  = interpcurve10(a_connect{1,1},c_connect{1,1},a_grid_,max_card);
p_equ_1(:,1,:)  = interpcurve10(a_connect{1,1},p_connect{1,1},a_grid_,max_card);

%%

% ================================================
% Period T - 2
% ================================================

% initialize endogenous coordinates
a_EGM_2        = cell(numb_a_gridpoints,1);
c_EGM_2        = cell(numb_a_gridpoints,1);
p_min_EGM_2    = cell(numb_a_gridpoints,1);
p_max_EGM_2    = cell(numb_a_gridpoints,1);
a_constr_EGM_2 = cell(numb_a_gridpoints,1);
c_constr_EGM_2 = cell(numb_a_gridpoints,1);
p_constr_EGM_2 = cell(numb_a_gridpoints,1);
kappa_2        = cell(numb_a_gridpoints,1);
mask_overlap_unconstr = cell(numb_a_gridpoints,1);
numb_equ       = NaN*zeros(numb_a_gridpoints,1);

MUc_1 = NaN*zeros(size(c_pol_1));
MUh_1 = NaN*zeros(size(c_pol_1));

for iap = 1:numb_a_gridpoints
    for ico = 1:max_card
        if c_pol_1(iap,1,ico) > 0 % calculate partial derivatives of the utility function
            MUc_1(iap,1,ico) =     theta  * (c_pol_1(iap,1,ico)^(theta*(1 -sigma_)-1)) * h_T_1^((1-theta)*(1-sigma_));
            MUh_1(iap,1,ico) =  (1-theta) *  c_pol_1(iap,1,ico)^(theta*(1 -sigma_))    * h_T_1^((1-theta)*(1-sigma_) -1);
        end
    end
end

for iap = 1:numb_a_gridpoints
    
    % detecting the number of equilibria
    numb_equ_i = sum(c_pol_1(iap,:,:) > 0, 3);
    
    if numb_equ_i > 0
        
        if numb_equ_i == 1
            size_simplices = 1;
        else
            size_simplices = numb_muc_steps;
        end
        
        % preparing cell-entry space to fit all possible confidence weightings
        % the parameterizations of weightings to be considered, depend on the number of equilibria
        a_EGM_2_i =        NaN*zeros(1,size_simplices);
        c_EGM_2_i =        NaN*zeros(1,size_simplices);
        p_min_EGM_2_i =    NaN*zeros(1,size_simplices);
        p_max_EGM_2_i =    NaN*zeros(1,size_simplices);
        a_constr_EGM_2_i = NaN*zeros(1,size_simplices);
        c_constr_EGM_2_i = NaN*zeros(1,size_simplices);
        p_constr_EGM_2_i = NaN*zeros(1,size_simplices);
        kappa_2_i =        NaN*zeros(1,size_simplices);
        
        if numb_equ_i == 1
            unconstr_numer = reshape((MUc_1(iap,1,1:numb_equ_i).*p_equ_1(iap,1,1:numb_equ_i) + MUh_1(iap,1,1:numb_equ_i)),1,numb_equ_i)*1.0*beta_;
            unconstr_denom = reshape( MUc_1(iap,1,1:numb_equ_i)*(1+r_T_1)                                                    ,1,numb_equ_i)*1.0*beta_;
            % evaluating FOC for the unconstrained case
            unconstr_FOC = unconstr_numer/unconstr_denom;
            
            if unconstr_FOC > -a_grid_(iap)*(1 + r_T_1)/(miu*h_T_1)
                % equilibrium for preceding states with non-binding constraints
                % -------------------------------------------------------------
                p_min_EGM_2_i = unconstr_FOC;
                p_max_EGM_2_i = p_min_EGM_2_i;
                c_EGM_2_i = (unconstr_denom/(theta*h_T_2^((1-theta)*(1-sigma_)))).^(1/(theta*(1 - sigma_) - 1));
                a_EGM_2_i = (c_EGM_2_i + a_grid_(iap) - y_T_2)/(1 + r_T_2);
                
            end % of if on existence of unconstrained case, mapping into a_prime
            
            if a_grid_(iap) < 0 && unconstr_FOC >= -a_grid_(iap)*(1 + r_T_1)/(miu*h_T_1)
                % equilibrium for preceding states with binding constraints
                % ---------------------------------------------------------
                p_constr_EGM_2_i = -a_grid_(iap)*(1 + r_T_1)/(miu*h_T_1);
                
                kappa_2_i = 1/(1.0 + r_T_1 - miu)*(unconstr_numer/p_constr_EGM_2_i - unconstr_denom);
                
                c_constr_EGM_2_i = (1/(theta*h_T_2^((1-theta)*(1-sigma_)))*unconstr_denom + kappa_2_i*(1 + r_T_1)/(theta*h_T_2^((1-theta)*(1-sigma_)))).^(1/(theta*(1 - sigma_) - 1));
                a_constr_EGM_2_i = 1.0/(1.0 + r_T_2)*(c_constr_EGM_2_i + a_grid_(iap) - y_T_2);
                
            end % of if on existence of constrained case (also requires negative financial assets)
        else
            unconstr_numer_C = reshape((MUc_1(iap,1,1:numb_equ_i).*p_equ_1(iap,1,1:numb_equ_i) + MUh_1(iap,1,1:numb_equ_i)),1,numb_equ_i)*beta_;
            unconstr_denom_C = reshape( MUc_1(iap,1,1:numb_equ_i)*(1+r_T_1)                                                    ,1,numb_equ_i)*beta_;
            
            if a_grid_(iap) < 0
                p_constr_EGM_2_i_C = -a_grid_(iap)*(1 + r_T_1)/(miu*h_T_1);
                kappa_2_i_C = 1/(1.0 + r_T_1 - miu)*(unconstr_numer_C/p_constr_EGM_2_i_C - unconstr_denom_C);
                pure_feasible = kappa_2_i_C >= 0;
            else
                kappa_2_i_C = zeros(size(unconstr_numer_C));
                pure_feasible = unconstr_numer_C > - Inf;
            end
            
            belief_cutoffs =         NaN*zeros(numb_equ_i,numb_equ_i);
            unconstr_denom_cutoffs = NaN*zeros(numb_equ_i,numb_equ_i);
            kappa_2_cutoffs =        NaN*zeros(numb_equ_i,numb_equ_i);
            c_EGM_2_cutoffs =        NaN*zeros(numb_equ_i,numb_equ_i);
            a_EGM_2_cutoffs =        NaN*zeros(numb_equ_i,numb_equ_i);
            is_pure_cutoffs =        zeros(numb_equ_i,numb_equ_i) > ones(numb_equ_i,numb_equ_i);
            
            
            for ip1 = 1:numb_equ_i
                if pure_feasible(ip1)
                    belief_cutoffs(ip1,ip1) = 1;
                    is_pure_cutoffs(ip1,ip1) = true;
                    unconstr_denom_cutoffs(ip1,ip1) = MUc_1(iap,1,ip1)*(1+r_T_1)*beta_;
                    c_EGM_2_cutoffs(ip1,ip1) = (unconstr_denom_cutoffs(ip1,ip1)/(theta*h_T_2^((1-theta)*(1-sigma_)))).^(1/(theta*(1 - sigma_) - 1));
                    a_EGM_2_cutoffs(ip1,ip1) = (c_EGM_2_cutoffs(ip1,ip1) + a_grid_(iap) - y_T_2)/(1 + r_T_2);
                    kappa_2_cutoffs(ip1,ip1) = kappa_2_i_C(ip1);
                    
                end
                for ip2 = 1:numb_equ_i
                    if ip2 < ip1
                        if xor(pure_feasible(ip1),pure_feasible(ip2))
                            if a_grid_(iap) < 0
                                belief_cutoffs(ip1,ip2) =  kappa_2_i_C(ip2)/( kappa_2_i_C(ip2) - kappa_2_i_C(ip1));
                            end
                            belief_order_natural = zeros(1,numb_equ_i);
                            belief_order_natural(ip1) = belief_cutoffs(ip1,ip2);
                            belief_order_natural(ip2) = 1 - belief_cutoffs(ip1,ip2);
                            
                            unconstr_denom_cutoffs(ip1,ip2) = reshape( MUc_1(iap,1,1:numb_equ_i)*(1+r_T_1),1,numb_equ_i)*belief_order_natural'*beta_;
                            c_EGM_2_cutoffs(ip1,ip2) = (unconstr_denom_cutoffs(ip1,ip2)/(theta*h_T_2^((1-theta)*(1-sigma_)))).^(1/(theta*(1 - sigma_) - 1));
                            a_EGM_2_cutoffs(ip1,ip2) = (c_EGM_2_cutoffs(ip1,ip2) + a_grid_(iap) - y_T_2)/(1 + r_T_2);
                            
                            kappa_2_cutoffs(ip1,ip2) = kappa_2_i_C*belief_order_natural';
                        end
                    end
                end
            end
            
            % get minimum and maximum of object linear in expectations which maps into consumption
            
            min_unconstr_denom_cutoff = min(min(unconstr_denom_cutoffs));
            max_unconstr_denom_cutoff = max(max(unconstr_denom_cutoffs));
            
            min_constr_gain_cutoff    = min(min(unconstr_denom_cutoffs + kappa_2_cutoffs*(1 + r_T_1)));
            max_constr_gain_cutoff    = max(max(unconstr_denom_cutoffs + kappa_2_cutoffs*(1 + r_T_1)));
            
            unconstr_denom_grid = linspace(min_unconstr_denom_cutoff,max_unconstr_denom_cutoff,numb_muc_steps);
            constr_gain_grid =    linspace(min_constr_gain_cutoff,   max_constr_gain_cutoff,   numb_muc_steps);
            
            % get consumption and endogenous asset gridpoint
            c_EGM_2_i = (unconstr_denom_grid/(theta*h_T_2^((1-theta)*(1-sigma_)))).^(1/(theta*(1 - sigma_) - 1));
            a_EGM_2_i = (c_EGM_2_i + a_grid_(iap) - y_T_2)/(1 + r_T_2);
            p_collect_EGM_2 = NaN*zeros(4,size(c_EGM_2_i,2));
            
            if a_grid_(iap) < 0
                if any(~isnan(constr_gain_grid))
                    p_constr_EGM_2_i = ones(1,size(constr_gain_grid,2))*(-a_grid_(iap)*(1 + r_T_1)/(miu*h_T_1));
                end
                c_constr_EGM_2_i = (1/(theta*h_T_2^((1-theta)*(1-sigma_)))*(constr_gain_grid)).^(1/(theta*(1 - sigma_) - 1));
                a_constr_EGM_2_i = (c_constr_EGM_2_i + a_grid_(iap) - y_T_2)/(1 + r_T_2);
            else
                p_constr_EGM_2_i = NaN;
                c_constr_EGM_2_i = NaN;
                a_constr_EGM_2_i = NaN;
                
            end % of if on negative successor asset value
            
            write_row_p = 1;
            
            % detect edges of subset of belief-simplex for which equilibrium exists
            % and compute extrema for the price
            
            for icut1 = 1:numb_equ_i
                for icut2 = 1:numb_equ_i
                    
                    if ~isnan(belief_cutoffs(icut1,icut2))
                        for icheck_row = icut1:numb_equ_i
                            
                            if icheck_row == icut1
                                start_col_check = icut2 + 1;
                            else
                                start_col_check = 1;
                            end
                            
                            for icheck_col = start_col_check:numb_equ_i
                                
                                if (~isnan(belief_cutoffs(icheck_row,icheck_col))...
                                        && ( icheck_row == icut1 || icheck_col == icut2))...
                                        || ...
                                        (is_pure_cutoffs(icut1,icut2) && is_pure_cutoffs(icheck_row,icheck_col))
                                    
                                    write_row_p = write_row_p + 1;
                                    
                                    % determine convex weight between corners of given edge
                                    weight1 = (unconstr_denom_grid - unconstr_denom_cutoffs(icheck_row,icheck_col))/...
                                        (unconstr_denom_cutoffs(icut1,icut2) - unconstr_denom_cutoffs(icheck_row,icheck_col));
                                    
                                    weight1(weight1 < 0 | weight1 > 1) = NaN;
                                    
                                    belief1_order_natural = zeros(1,numb_equ_i);
                                    if is_pure_cutoffs(icut1,icut2)
                                        belief1_order_natural(icut1) = 1;
                                    else
                                        belief1_order_natural(icut1) = belief_cutoffs(icut1,icut2);
                                        belief1_order_natural(icut2) = 1 - belief_cutoffs(icut1,icut2);
                                    end
                                    
                                    belief2_order_natural = zeros(1,numb_equ_i);
                                    if is_pure_cutoffs(icheck_row,icheck_col)
                                        belief2_order_natural(icheck_row) = 1;
                                    else
                                        belief2_order_natural(icheck_row) = belief_cutoffs(icheck_row,icheck_col);
                                        belief2_order_natural(icheck_col) = 1 - belief_cutoffs(icheck_row,icheck_col);
                                    end
                                    
                                    belief_edge_intersection = kron(weight1,belief1_order_natural') + kron((1 - weight1),belief2_order_natural');
                                    
                                    % determine extremum price
                                    
                                    unconstr_numer = reshape((MUc_1(iap,1,1:numb_equ_i).*p_equ_1(iap,1,1:numb_equ_i) + MUh_1(iap,1,1:numb_equ_i)),1,numb_equ_i)*belief_edge_intersection*beta_;
                                    unconstr_denom = reshape( MUc_1(iap,1,1:numb_equ_i)*(1+r_T_1)                                                    ,1,numb_equ_i)*belief_edge_intersection*beta_;
                                    % evaluating FOC for the unconstrained case
                                    p_collect_EGM_2(write_row_p,:) = unconstr_numer./unconstr_denom;
                                    
                                end % of if
                                
                            end % of for over cols checked for an edge
                        end % of for over rows checked for an edge
                        
                    end % of if on being at some corner of existence range on subset of simplex
                    
                end % of for over rows providing first element for edge
            end % of for over cols providing first element of edge
            
            p_min_EGM_2_i = min(p_collect_EGM_2);
            p_max_EGM_2_i = max(p_collect_EGM_2);
        end
        
        a_EGM_2{iap,1} =        a_EGM_2_i;
        c_EGM_2{iap,1} =        c_EGM_2_i;
        p_min_EGM_2{iap,1} =    p_min_EGM_2_i;
        p_max_EGM_2{iap,1} =    p_max_EGM_2_i;
        a_constr_EGM_2{iap,1} = a_constr_EGM_2_i;
        c_constr_EGM_2{iap,1} = c_constr_EGM_2_i;
        p_constr_EGM_2{iap,1} = p_constr_EGM_2_i;
        kappa_2{iap,1} =        kappa_2_i;
        
        if numb_equ_i > 1
            mask_overlap_unconstr{iap,1} = a_EGM_2{iap,1} < max(a_constr_EGM_2{iap,1}) - sqrt(eps);
        end
        
    end % of if on existence of equilibrium over continuation state a_prime
    numb_equ(iap,1) = numb_equ_i;
    
end

% find A_T_2 on delimiting curve for constrained equilibrium prices
delimiting_constr_price = zeros(sum(numb_equ > 1),1)*NaN;
delimiting_constr_A_T_2 = zeros(sum(numb_equ > 1),1)*NaN;
delimiting_constr_successor_index = zeros(sum(numb_equ > 1),1)*NaN;
write_index_delimiting = 1;
for iap = 1:numb_a_gridpoints
    if numb_equ(iap,1) > 1
        [delimiting_constr_A_T_2(write_index_delimiting),i_delimit_max] = max(a_constr_EGM_2{iap,1});
        this_p_constr = p_constr_EGM_2{iap,1};
        delimiting_constr_price(write_index_delimiting) = this_p_constr(i_delimit_max);
        delimiting_constr_successor_index(write_index_delimiting) = iap;
        write_index_delimiting = write_index_delimiting + 1;
    end
end

mask_overlap_unconstr_A_T_2_p_min_T_2 = mask_overlap_unconstr;
for iap = 1:numb_a_gridpoints
    this_mask_overlap_unconstr = mask_overlap_unconstr{iap,1};
    if sum(this_mask_overlap_unconstr) >= 1
        this_a_EGM_2 = a_EGM_2{iap,1};
        this_p_min_EGM_2 = p_min_EGM_2{iap,1};
        check_delimiting = zeros(1,numel(this_mask_overlap_unconstr)) > 1;
        
        check_delimiting(this_mask_overlap_unconstr) = this_a_EGM_2(this_mask_overlap_unconstr) <= ...
            interpconnected(delimiting_constr_price,delimiting_constr_A_T_2,...
            delimiting_constr_successor_index,this_p_min_EGM_2(this_mask_overlap_unconstr));
        
        mask_overlap_unconstr_A_T_2_p_min_T_2{iap,1} = this_mask_overlap_unconstr & check_delimiting;
        
    end
end

mask_overlap_unconstr_A_T_2_p_max_T_2 = mask_overlap_unconstr;
for iap = 1:numb_a_gridpoints
    this_mask_overlap_unconstr = mask_overlap_unconstr{iap,1};
    if sum(this_mask_overlap_unconstr) >= 1
        this_a_EGM_2 = a_EGM_2{iap,1};
        this_p_max_EGM_2 = p_max_EGM_2{iap,1};
        check_delimiting = zeros(1,numel(this_mask_overlap_unconstr)) > 1;
        
        check_delimiting(this_mask_overlap_unconstr) = this_a_EGM_2(this_mask_overlap_unconstr) <= ...
            interpconnected(delimiting_constr_price,delimiting_constr_A_T_2,...
            delimiting_constr_successor_index,this_p_max_EGM_2(this_mask_overlap_unconstr));
        
        mask_overlap_unconstr_A_T_2_p_max_T_2{iap,1} = this_mask_overlap_unconstr & check_delimiting;
        
    end
end

a_grid_cell = cell(size(c_EGM_2));
a_grid_constr_cell = cell(size(c_constr_EGM_2));
c_NORM_EGM_2 = cell(size(c_EGM_2));
for ip = 1:size(c_EGM_2,1)
    c_NORM_EGM_2{ip,1} = c_EGM_2{ip,1}/period_years;
    a_grid_cell{ip,1} = a_grid_(ip)*ones(size(c_EGM_2{ip,1}));
end
c_constr_NORM_EGM_2 = cell(size(c_constr_EGM_2));
for ip = 1:size(c_constr_EGM_2,1)
    c_constr_NORM_EGM_2{ip,1} = c_constr_EGM_2{ip,1}/period_years;
    a_grid_constr_cell{ip,1} = a_grid_(ip)*ones(size(c_constr_EGM_2{ip,1}));
end

% -----------------------------------------------------------------------------------------------------
% marker definitions for scatter plots
m_unc{1,1} = 'Marker'; m_unc{1,2} = '.'; m_unc{1,3} = 'MarkerEdgeColor'; m_unc{1,4} = 'g';
m_unc{1,5} = 'MarkerFaceColor'; m_unc{1,6} = 'g'; m_unc{1,7} = 'MarkerSize';  m_unc{1,8} = 0.3; % unconstrained, successor unique

m_unc{2,1} = 'Marker'; m_unc{2,2} = 'o'; m_unc{2,3} = 'MarkerEdgeColor'; m_unc{2,4} = 'g';
m_unc{2,5} = 'MarkerFaceColor'; m_unc{2,6} = 'g'; m_unc{2,7} = 'MarkerSize';  m_unc{2,8} = 0.3; % unconstrained, successor multiple

m_con{1,1} = 'Marker'; m_con{1,2} = '.'; m_con{1,3} = 'MarkerEdgeColor'; m_con{1,4} = 'r';
m_con{1,5} = 'MarkerFaceColor'; m_con{1,6} = 'r'; m_con{1,7} = 'MarkerSize';  m_con{1,8} = 0.3; % constrained,   successor unique

m_con{2,1} = 'Marker'; m_con{2,2} = 'o'; m_con{2,3} = 'MarkerEdgeColor'; m_con{2,4} = 'r';
m_con{2,5} = 'MarkerFaceColor'; m_con{2,6} = 'r'; m_con{2,7} = 'MarkerSize';  m_con{2,8} = 0.3; % constrained,   successor multiple

m_restr{1,1} = 'Marker'; m_restr{1,2} = 'p'; m_restr{1,3} = 'MarkerEdgeColor'; m_restr{1,4} = 'g';
m_restr{1,5} = 'MarkerFaceColor'; m_restr{1,6} = 'g'; m_restr{1,7} = 'MarkerSize';  m_restr{1,8} =4; % unconstrained, successor unique

m_restr{2,1} = 'Marker'; m_restr{2,2} = 'o'; m_restr{2,3} = 'MarkerEdgeColor'; m_restr{2,4} = 'g';
m_restr{2,5} = 'MarkerFaceColor'; m_restr{2,6} = 'g'; m_restr{2,7} = 'MarkerSize';  m_restr{2,8} =2; % unconstrained, successor multiple

m_restr{3,1} = 'Marker'; m_restr{3,2} = 'p'; m_restr{3,3} = 'MarkerEdgeColor'; m_restr{3,4} = 'r';
m_restr{3,5} = 'MarkerFaceColor'; m_restr{3,6} = 'r'; m_restr{3,7} = 'MarkerSize';  m_restr{3,8} =4; % constrained, successor unique

m_restr{4,1} = 'Marker'; m_restr{4,2} = 'o'; m_restr{4,3} = 'MarkerEdgeColor'; m_restr{4,4} = 'r';
m_restr{4,5} = 'MarkerFaceColor'; m_restr{4,6} = 'r'; m_restr{4,7} = 'MarkerSize';  m_restr{4,8} =2; % constrained, successor multiple

% USER: specify m_OL{1,2} = 'x' to check results before coloring overlap with m_OL{1,2} = 'o'
m_OL{1,1} = 'Marker'; m_OL{1,2} = 'o'; m_OL{1,3} = 'MarkerEdgeColor'; m_OL{1,4} = 'b';
m_OL{1,5} = 'MarkerFaceColor'; m_OL{1,6} = 'b'; m_OL{1,7} = 'MarkerSize';  m_OL{1,8} =1.2; % unconstrained, successor multiple

% ---------------------------------------------------------------------------------------------------------------------------

% % figure(2446); hold on;
% % connect_neighbors_plot(a_constr_EGM_2,a_grid_constr_cell,m_con)
% % connect_neighbors_plot(a_EGM_2,a_grid_cell,m_unc)
% % for ip = 1:numel(a_grid_)
% %     if numel(a_EGM_2{ip,1}) > 0 && numb_equ(ip,1) > 1
% %         xv = a_EGM_2{ip,1};
% %         mask_overlap_v = mask_overlap_unconstr{ip,1};
% %         yv = a_grid_cell{ip,1};
% %         for i = 1:numel(a_EGM_2{ip,1})
% %             if mask_overlap_v(i)
% %                 line(xv(i),yv(i),'LineStyle','none',m_OL{1,:});
% %             end
% %         end
% %     end
% % end
% % 
% % xlabel('A_{T-2}')
% % ylabel('successor-state, A_{T-1}')
% % hold off;
% % nice_plot_medium2;
% % axis([-8.0,1.0,-8.0,0.3]);
% % set(gcf,'PaperUnits','centimeters','PaperPosition',[0 0 Plot_Width 0.75*Plot_Width])
% % %  print -r864 -painters -dpng bench_Aprime_T_2.png

% ---------------------------------------------------------------------------------
figure(2447); hold on;
connect_neighbors_plot(a_constr_EGM_2,c_constr_NORM_EGM_2,m_con)
connect_neighbors_plot(a_EGM_2,c_NORM_EGM_2,m_unc)
for ip = 1:numel(a_grid_)
    if numel(a_EGM_2{ip,1}) > 0 && numb_equ(ip,1) > 1
        xv = a_EGM_2{ip,1};
        mask_overlap_v = mask_overlap_unconstr{ip,1};
        yv = c_NORM_EGM_2{ip,1};
        for i = 1:numel(a_EGM_2{ip,1})
            if mask_overlap_v(i)
                line(xv(i),yv(i),'LineStyle','none',m_OL{1,:});
            end
        end
    end
end

xlabel('A_{T-2}')
ylabel('cons. (annual), c_{T-2}')
hold off;
nice_plot_medium2;
axis([-8.0,1.0,0.3,1.3]);
set(gcf,'PaperUnits','centimeters','PaperPosition',[0 0 Plot_Width 0.75*Plot_Width])
%  print -r864 -painters -dpng bench_c_T_2.png

% --------------------------------------------------------------------------------
figure(2448); hold on;
connect_neighbors_plot(a_constr_EGM_2,p_constr_EGM_2,m_con)
connect_neighbors_plot(a_EGM_2,p_max_EGM_2,m_unc)
connect_neighbors_plot(a_EGM_2,p_min_EGM_2,m_unc)
for ip = 1:numel(a_grid_)
    if numel(a_EGM_2{ip,1}) > 0 && numb_equ(ip,1) > 1
        xv = a_EGM_2{ip,1};
        mask_overlap_v = mask_overlap_unconstr_A_T_2_p_min_T_2{ip,1};
        yv = p_min_EGM_2{ip,1};
        for i = 1:numel(a_EGM_2{ip,1})
            if mask_overlap_v(i)
                line(xv(i),yv(i),'LineStyle','none',m_OL{1,:});
            end
        end
    end
end
for ip = 1:numel(a_grid_)
    if numel(a_EGM_2{ip,1}) > 0 && numb_equ(ip,1) > 1
        xv = a_EGM_2{ip,1};
        mask_overlap_v = mask_overlap_unconstr_A_T_2_p_max_T_2{ip,1};
        yv = p_max_EGM_2{ip,1};
        for i = 1:numel(a_EGM_2{ip,1})
            if mask_overlap_v(i)
                line(xv(i),yv(i),'LineStyle','none',m_OL{1,:});
            end
        end
    end
end

xlabel('financial state, A_{T-2}')
ylabel('house price, p_{T-2}')
hold off;
nice_plot_medium2;
axis([-8.0,1.0,0,18.5]);
set(gcf,'PaperUnits','centimeters','PaperPosition',[0 0 Plot_Width 0.75*Plot_Width])
%  print -r864 -painters -dpng bench_p_T_2.png





