% main_multiplicity_regions.m
%
% Computes multiplicity regions
% as reported in Section 3 of the paper
% 'Household Debt and Crises of Confidence', Quantitative Economics,
%
% Thomas Hintermaier, thomas.hintermaier@uni-bonn.de and
% Winfried Koeniger,  winfried.koeniger@unisg.ch
%
% March 16, 2018
% ======================================================

clearvars;

plot_settings_collateral;

% Algorithm parameters
% ====================
max_card = 4;            % maximum number of equilibria per state
numb_muc_steps = 200;    % number of steps between minimum and maximum marginal utility in line with equilibrium existence

% Model parameters
% ================
period_years = 10;

% the interest rate applicable during the final stage, i.e., to assets a_t, t >= T+1
r     = 1.04^period_years - 1; % interest rate on savings, BENCHMARK

beta_ = 1/(1 + r);

r_T   = 1.04^period_years - 1;    % interest rate on a_T
r_T_1 = 1.04^period_years - 1;  % interest rate on a_T_1
r_T_2 = 1.04^period_years - 1;  % interest rate on a_T_2

sigma_  = 10.0;     % overall risk aversion, BENCHMARK

theta = 0.70;       % Cobb-Douglas weight on non-durable consumption

miu = 0.80;

y_T   = 1.0*period_years; % the endowment for the entire final stage, per period
y_T_1 = 1.0*period_years;
y_T_2 = 1.0*period_years;

% Create the grid on the state space
% ==================================

% h is housing
h_T   = 1.0;
h_T_1 = 1.0;
h_T_2 = 1.0;

% a_min is the lowest gridpoint at which c_T > 0
% despite ensuring positive consumption in period T,
% the endowment pattern in earlier periods might require
% lower values for the bound in preceeding periods
% USER: adjust grid in T - 1 and T - 2 if objects obtained display gaps
% (because of a grid specification which would discard some of the equilibria)

a_min = -y_T*(1 + r)/(r*(1 + r_T)) + eps;
a_max = 20*y_T;

a_small = 0.000000001;
numb_a_minus_gridpoints = 5500;
numb_a_plus_gridpoints  = 1500;

a_grid_minus = linspace(a_min  ,-a_small,numb_a_minus_gridpoints );
a_grid_plus  = linspace(a_small,   a_max,numb_a_plus_gridpoints  );

a_grid_ = [a_grid_minus,a_grid_plus]';  % set up exogenous grid over continuation-state (next-period state)
numb_a_gridpoints = length(a_grid_);

sigma_vec = 1.01:0.05:20.1;
miu_vec   = 0.01:0.005:1.01;
% USER, feel free to comment in the specification with a finer resolution for the parameter grid,
% but note that this would increase computing time considerably.
% % sigma_vec = 1.01:0.01:20.1;
% % miu_vec  = 0.01:0.001:1.01;

[SIGMA,MIU] = meshgrid(sigma_vec,miu_vec);

% evaluating the condition for backward-bendingness at the kink
% evaluating on a grid of parameterizations
TEST_greater_minus_1 = (1./(1 + r_T - MIU)).*((beta_*(1 + r_T)).^(1./(theta*(1 - SIGMA) - 1)))*r.*...
    (  ((1 + r_T)^2/(1 + r)*theta./(theta*(1 - SIGMA) - 1)).*...
    (1 - SIGMA + (1 + r_T)./(MIU*(1 - theta))) - MIU*(1 + r_T)/(1 + r)  );

level_v = [-1,-1];

TEST_at_least_2_eqilib = zeros(size(TEST_greater_minus_1));

figure(101);
contourf(SIGMA,MIU,TEST_greater_minus_1,level_v,'k','LineWidth',2);
colormap(hot);
xlabel('\sigma')
ylabel('\mu')
title('Region of parameters for backward-bendingness at kink')
nice_plot_medium2;

for i_miu = 1:length(miu_vec)
    for j_sig = 1:length(sigma_vec)
        
        if mod(10*(i_miu*length(sigma_vec) - j_sig + 1),length(miu_vec)*length(sigma_vec)) == 0
            disp(['Completed ',num2str((i_miu*length(sigma_vec) - j_sig + 1)/(length(miu_vec)*length(sigma_vec))),...
                ' of (miu,sigma) combinations.']);
        end
        
        miu = miu_vec(i_miu);
        sigma_ = sigma_vec(j_sig);
        
        % =============================================================================
        % Initialize equilibrium policy function and price function for terminal stage
        % =============================================================================
        c_pol = NaN*zeros(numb_a_gridpoints,1);
        p_equ = NaN*zeros(numb_a_gridpoints,1);
        % p_equ =     20*ones(numb_a_gridpoints,1);
        
        c_pol(:,1) = r*(1 + r_T)/(1 + r)*a_grid_ + y_T;
        p_equ(:,1) = (1 - theta)/theta*c_pol(:,1)/r;
        
        % ==============================================================
        % Intermediate period, T - 1
        % ==============================================================
        
        MUc = NaN*zeros(numb_a_gridpoints,1);
        MUh = NaN*zeros(numb_a_gridpoints,1);
        
        for iap = 1:numb_a_gridpoints
            if c_pol(iap,1) > 0
                % calculate partial derivatives of the utility function
                MUc(iap,1) =     theta  * (c_pol(iap,1)^(theta*(1 -sigma_)-1)) * h_T^((1-theta)*(1-sigma_));
                MUh(iap,1) =  (1-theta) *  c_pol(iap,1)^(theta*(1 -sigma_))    * h_T^((1-theta)*(1-sigma_) -1);
            end
        end
        
        unconstr_numer = (MUc.*p_equ + MUh)*1.0'*beta_;
        unconstr_denom =  MUc*(1+r_T)      *1.0'*beta_;
        % evaluating FOC for the unconstrained case
        unconstr_FOC = unconstr_numer./unconstr_denom;
        
        % initialize endogenous coordinates
        a_EGM =        NaN*zeros(numb_a_gridpoints,1);
        c_EGM =        NaN*zeros(numb_a_gridpoints,1);
        p_EGM =        NaN*zeros(numb_a_gridpoints,1);
        a_constr_EGM = NaN*zeros(numb_a_gridpoints,1);
        c_constr_EGM = NaN*zeros(numb_a_gridpoints,1);
        p_constr_EGM = NaN*zeros(numb_a_gridpoints,1);
        kappa =        NaN*zeros(numb_a_gridpoints,1);
        
        
        for iap = 1:numb_a_gridpoints
            
            if unconstr_FOC(iap,1) > -a_grid_(iap)*(1 + r_T)/(miu*h_T)
                % equilibrium for preceding states with non-binding constraints
                % -------------------------------------------------------------
                p_EGM(iap,1) = unconstr_FOC(iap,1);
                c_EGM(iap,1) = (unconstr_denom(iap,1)/(theta* h_T_1^((1-theta)*(1-sigma_)))).^(1/(theta*(1 - sigma_) - 1));
                a_EGM(iap,1) = (c_EGM(iap,1) + a_grid_(iap) - y_T_1)/(1 + r_T_1);
                
            end % of if on existence of unconstrained case, mapping into a_prime
            
            if a_grid_(iap) < 0 && unconstr_FOC(iap,1) >= -a_grid_(iap)*(1 + r_T)/(miu*h_T)
                % equilibrium for preceding states with binding constraints
                % ---------------------------------------------------------
                p_constr_EGM(iap,1) = -a_grid_(iap)*(1 + r_T)/(miu*h_T);
                
                kappa(iap,1) = 1/(1.0 + r_T - miu)*(unconstr_numer(iap,1)./p_constr_EGM(iap,1) - unconstr_denom(iap,1));
                
                c_constr_EGM(iap,1) = ( (unconstr_denom(iap,1) + kappa(iap,1)*(1 + r_T))/(theta*h_T_1^((1-theta)*(1-sigma_)))).^(1/(theta*(1 - sigma_) - 1));
                a_constr_EGM(iap,1) = 1.0/(1.0 + r_T_1)*(c_constr_EGM(iap,1) + a_grid_(iap) - y_T_1);
                
            end % of if on existence of constrained case (also requires negative financial assets)
            
        end
        
        
        % detect vertices and use them to locate kinks in policies
        p_xp0 = -(1 + r_T)*a_grid_/(miu*h_T);
        ap_cutoff = NaN*zeros(1,1);
        a_cutoff_EGM = NaN*zeros(1,1);
        c_cutoff_EGM = NaN*zeros(1,1);
        p_cutoff_EGM = NaN*zeros(1,1);
        
        
        sel1_not_NaN = ~isnan(unconstr_FOC(:,1));
        ap_cutoff(1) = interp1(unconstr_FOC(sel1_not_NaN,1) - p_xp0(sel1_not_NaN),a_grid_(sel1_not_NaN),0);
        if ap_cutoff(1) < 0
            p_cutoff_EGM(1) = -(1 + r_T)*ap_cutoff(1)/(miu*h_T);
            c_cutoff_EGM(1) = (1/(theta*h_T_1^((1-theta)*(1-sigma_)))*interp1(a_grid_(sel1_not_NaN),unconstr_denom(sel1_not_NaN,1),ap_cutoff(1)) ).^(1/(theta*(1 - sigma_) - 1));
            a_cutoff_EGM(1) = 1.0/(1.0 + r_T_1)*(c_cutoff_EGM(1) + ap_cutoff(1) - y_T_1);
        end        
        
        ap_cutoff_analytic = -miu*(1 - theta)/(theta*(1 + r_T) + miu*(1 - theta))*(1 + r)/(r*(1 + r_T))*y_T;
        
        a_under_arrow_T_1 = (ap_cutoff_analytic*(1 + (beta_*(1 + r_T))^(1/(theta*(1 - sigma_) - 1))*r*(1 + r_T)/(1 + r))...
            + (beta_*(1 + r_T))^(1/(theta*(1 - sigma_) - 1))*y_T - y_T_1)...
            /(1 + r_T_1);        
        
        % splicing the relevant equilibrium curves, ensuring connectedness
        sel_none_NaN_constr = ~isnan(a_constr_EGM) & ~isnan(c_constr_EGM) & ~isnan(p_constr_EGM);
        sel_none_NaN        = ~isnan(a_EGM)        & ~isnan(c_EGM)        & ~isnan(p_EGM);
        
        a_connect = cell(1,1);
        c_connect = cell(1,1);
        p_connect = cell(1,1);
        
        sort_a_c_p_constr = sortrows([a_constr_EGM(sel_none_NaN_constr(:,1),1),c_constr_EGM(sel_none_NaN_constr(:,1),1),...
            p_constr_EGM(sel_none_NaN_constr(:,1),1),     a_grid_(sel_none_NaN_constr(:,1))],-4);
        
        a_connect{1,1} = [sort_a_c_p_constr(:,1);a_cutoff_EGM(1);a_EGM(sel_none_NaN(:,1),1)];
        c_connect{1,1} = [sort_a_c_p_constr(:,2);c_cutoff_EGM(1);c_EGM(sel_none_NaN(:,1),1)];
        p_connect{1,1} = [sort_a_c_p_constr(:,3);p_cutoff_EGM(1);p_EGM(sel_none_NaN(:,1),1)];
        
        clear sort_a_c_p_constr        
        
        if abs(a_cutoff_EGM - a_under_arrow_T_1) < 0.0001 % safety check
            
            % getting (potentially multiple) equilibrium policies and prices on state-grid
            c_pol_1 = NaN*zeros(1,1,max_card);
            
            A_T_1_interp = a_under_arrow_T_1 + 0.0001; % just to the right of kink,
            % a bit of safety to make sure check does cover unconstrained part            
            
            c_pol_1(1,1,:)  = interpcurve10(a_connect{1,1},c_connect{1,1},A_T_1_interp,max_card);            
            
        end
        
        numb_equ_kink_ij = sum(c_pol_1(1,:,:) > 0, 3);
        
        TEST_at_least_2_eqilib(i_miu,j_sig) = numb_equ_kink_ij;
        
    end
end

level_v2 = [2,2];

figure(1033);
contourf(SIGMA,MIU,TEST_at_least_2_eqilib,level_v2,'k-','LineWidth',2);
% colormap(hot);
hold on;
[~,h] = contourf(SIGMA,MIU,TEST_greater_minus_1,level_v,'k--','LineWidth',2);
% colormap(summer);
colormap(autumn);
ch = get(h,'child'); alpha(ch,0.5)
hold off;
xlabel('utility curvature,   \sigma')
ylabel('loan-to-value ratio,   \mu')
% nice_plot_medium2;
axis([0,20,0,1]);
set(gca,'XTick',0:5:20);
set(gca,'YTick',0:0.2:1);




