% main_policies_and_demand.m
% 
% Computes household policy functions and
% (excess) demand, as reported in Section 3 of the paper
% 'Household Debt and Crises of Confidence', Quantitative Economics,
%
% Thomas Hintermaier, thomas.hintermaier@uni-bonn.de and
% Winfried Koeniger,  winfried.koeniger@unisg.ch
%
% March 16, 2018
% ======================================================

clearvars;

% Model parameters
% ================
period_years = 10;

% the interest rate applicable during the final stage, i.e., to assets a_t, t >= T+1
r     = 1.04^period_years - 1; % interest rate on savings, BENCHMARK 

beta_ = 1/(1 + r);

r_T   = 1.04^period_years - 1;    % interest rate on a_T
r_T_1 = 1.04^period_years - 1;  % interest rate on a_T_1

sigma_  = 10.0;     % overall risk aversion, BENCHMARK

theta = 0.70; %     % Cobb-Douglas weight on non-durable consumption

miu = 0.80;

y_T   = 1.0*period_years; % the endowment for the entire final stage, per period
y_T_1 = 1.0*period_years;

h_initial = 1.0; % initial   housing stock in period T-1
h_supply  = 1.0; % supply of housing stock

% Create the grid 
p_min = 0.0001;
p_max = 10;
numb_p_gridpoints = 1200;

constant_final_price = 7.424;

p_grid_ = linspace(p_min,p_max,numb_p_gridpoints);

% the additional grid for individual housing stocks is used for the constrained case
h_min = 0.0001;
h_max = 5*h_supply;
numb_h_gridpoints = 1900;
h_grid_ = linspace(h_min,h_max,numb_h_gridpoints);

% a_min is the lowest gridpoint at which c_T > 0
% despite ensuring positive consumption in period T,
% the endowment pattern in earlier periods might require
% lower values for the bound in preceeding periods
% USER: adjust grid in T - 1 and T - 2 if objects obtained display gaps
% (because of a grid specification which would discard some of the equilibria)

a_min = -y_T*(1 + r)/(r*(1 + r_T)) + 0.000001;
a_max = 1.5*y_T;

a_small = 0.000001;
numb_a_minus_gridpoints = 5500;
numb_a_plus_gridpoints  = 1100;

a_grid_minus = linspace(a_min  ,-a_small,numb_a_minus_gridpoints );
a_grid_plus  = linspace(a_small,   a_max,numb_a_plus_gridpoints  );

a_grid_ = [a_grid_minus,a_grid_plus]';  % set up exogenous grid over continuation-state (next-period state)
numb_a_gridpoints = length(a_grid_);

plot_prices_wanted = (5.715); % prices to be conditioned on for some plots

plot_asset_levels_wanted = [-5, -3.5, -2]; % asset levels, a_T_1, conditioned in for excess demand functions

plot_prices =      NaN*zeros(size(plot_prices_wanted));
plot_index_price = NaN*zeros(size(plot_prices_wanted));
for i_plot_p = 1:length(plot_prices_wanted)
    [min_dist_pi,plot_index_price(i_plot_p)] = min(abs(plot_prices_wanted(i_plot_p)- p_grid_));
    plot_prices(i_plot_p) = p_grid_(plot_index_price(i_plot_p));
end

plot_asset_levels = NaN*zeros(size(plot_asset_levels_wanted));
plot_index_asset =  NaN*zeros(size(plot_asset_levels_wanted));
for i_plot_a = 1:length(plot_asset_levels_wanted)
    [min_dist_ai,plot_index_asset(i_plot_a)] = min(abs(plot_asset_levels_wanted(i_plot_a)- a_grid_));
    plot_asset_levels(i_plot_a) = a_grid_(plot_index_asset(i_plot_a));  
end

% Initialize equilibrium policy function and price function for terminal stage
% ============================================================================
p_T = constant_final_price;
y_Tp1 = y_T;
c_Tp1 = r*a_grid_ + y_Tp1; % time T + 1 consumption, as a function of pre-exising financial assets in T + 1
h_Tp1 = (1 - theta)/(theta*r*p_T)*(r*a_grid_ + y_Tp1);    % time T + 1 pre-existing state of housing

% backward induction from T + 1 to T
% ==================================
c_T_unc = NaN*zeros(numb_a_gridpoints,numb_p_gridpoints);
h_T_unc = NaN*zeros(numb_a_gridpoints,numb_p_gridpoints);
a_T_unc = NaN*zeros(numb_a_gridpoints,numb_p_gridpoints);

for ip = 1:numb_p_gridpoints
    for iap = 1:numb_a_gridpoints
        if c_Tp1(iap) > 0 && h_Tp1(iap) > 0
            if (1 + r)*p_grid_(ip) > p_T
                c_T_unc(iap,ip) = (c_Tp1(iap)^(theta*(1 - sigma_) - 1)*h_Tp1(iap)^((1 - theta)*(1 - sigma_))*(theta/(1 - theta)*((1 + r)*p_grid_(ip) - p_T))^((1 - theta)*(1 - sigma_)))^(-1/sigma_);
                h_T_unc(iap,ip) = (1 - theta)/theta*c_T_unc(iap,ip)/((1 + r)*p_grid_(ip) - p_T);
                a_T_unc(iap,ip) = (a_grid_(iap) + p_T*h_Tp1(iap) + c_T_unc(iap,ip) - p_T*h_T_unc(iap,ip) - y_T)/(1 + r);
                if h_T_unc(iap,ip) < -(1 + r)/(miu*p_grid_(ip))*a_T_unc(iap,ip)
                      c_T_unc(iap,ip) = NaN;
                      h_T_unc(iap,ip) = NaN;
                      a_T_unc(iap,ip) = NaN;                      
                end     
            end
        end
    end
end

c_T_aTp1_hT = NaN*zeros(numb_a_gridpoints,numb_h_gridpoints); % c_T calculated for some a_T+1 and h_T
a_T_aTp1_hT = NaN*zeros(numb_a_gridpoints,numb_h_gridpoints); % a_T calculated for some a_T+1 and h_T

for ih = 1:numb_h_gridpoints
    for iap = 1:numb_a_gridpoints
        if c_Tp1(iap) > 0 && h_Tp1(iap) > 0
            c_T_aTp1_hT(iap,ih) = (c_Tp1(iap)^(theta*(1 - sigma_) - 1)*h_Tp1(iap)^((1 - theta)*(1 - sigma_))*h_grid_(ih)^(-(1 - theta)*(1 - sigma_)))^(1/(theta*(1 - sigma_) - 1));
            a_T_aTp1_hT(iap,ih) = (a_grid_(iap) + p_T*h_Tp1(iap) + c_T_aTp1_hT(iap,ih) - p_T*h_grid_(ih) - y_T)/(1 + r);
        end
    end
end

c_T_con = NaN*zeros(numb_h_gridpoints,numb_p_gridpoints);
h_T_con = NaN*zeros(numb_h_gridpoints,numb_p_gridpoints);
a_T_con = NaN*zeros(numb_h_gridpoints,numb_p_gridpoints);

for ih = 1:numb_h_gridpoints
    h_T_con(ih,:) = h_grid_(ih);
    a_T_con(ih,:) = -h_grid_(ih)*miu*p_grid_/(1 + r);
    c_T_con(ih,:) = interp1(a_T_aTp1_hT(:,ih),c_T_aTp1_hT(:,ih),a_T_con(ih,:));
end

% Period T - 1
% ==============================================================
MUc_unc =     theta  * c_T_unc.^(theta*(1 -sigma_)-1) .* h_T_unc.^((1-theta)*(1-sigma_));
unconstr_denom_unc =  MUc_unc*(1+r)*1.0'*beta_;
clear MUc_unc

a_EGM =        NaN*zeros(numb_a_gridpoints,numb_p_gridpoints);
c_EGM =        NaN*zeros(numb_a_gridpoints,numb_p_gridpoints);

for ip = 1:numb_p_gridpoints
    for iap = 1:numb_a_gridpoints
        
            % equilibrium for preceding states with non-binding constraints
            % -------------------------------------------------------------
            c_EGM(iap,ip) = (unconstr_denom_unc(iap,ip)/(theta* h_initial^((1-theta)*(1-sigma_))))^(1/(theta*(1 - sigma_) - 1));
            a_EGM(iap,ip) = (c_EGM(iap,ip) + a_T_unc(iap,ip) + p_grid_(ip)*(h_T_unc(iap,ip) - h_initial) - y_T_1)/(1 + r);
                            
    end
end
clear unconstr_denom_unc

MUc_con =     theta  * c_T_con.^(theta*(1 -sigma_)-1) .* h_T_con.^((1-theta)*(1-sigma_));
MUh_con =  (1-theta) * c_T_con.^(theta*(1 -sigma_))   .* h_T_con.^((1-theta)*(1-sigma_) -1);
unconstr_numer_con = (MUc_con*p_T + MUh_con)*1.0'*beta_;
unconstr_denom_con =  MUc_con*(1+r)*1.0'*beta_;
clear MUc_con
clear MUh_con

kappa_cand = NaN*zeros(size(unconstr_numer_con)); % candidates for kappa, verify that these are positive for constrained choices
for ip = 1:numb_p_gridpoints
    kappa_cand(:,ip) = 1/(1.0 + r - miu)*(unconstr_numer_con(:,ip)/p_grid_(ip) - unconstr_denom_con(:,ip));
    invalidate_vec = ~(kappa_cand(:,ip) > 0);
    c_T_con(invalidate_vec,ip) = NaN;
    h_T_con(invalidate_vec,ip) = NaN;
    a_T_con(invalidate_vec,ip) = NaN;
end
clear unconstr_numer_con

a_constr_EGM =        NaN*zeros(size(kappa_cand));
c_constr_EGM =        NaN*zeros(size(kappa_cand));
kappa =               NaN*zeros(size(kappa_cand));

for ih = 1:size(kappa_cand,1)
    for ip = 1:size(kappa_cand,2)
        if kappa_cand(ih,ip) > 0
            kappa(ih,ip) = kappa_cand(ih,ip);
            c_constr_EGM(ih,ip) = ( (unconstr_denom_con(ih,ip) + kappa(ih,ip)*(1 + r))/(theta*h_initial^((1-theta)*(1-sigma_))))^(1/(theta*(1 - sigma_) - 1));
            a_constr_EGM(ih,ip) = (c_constr_EGM(ih,ip) + a_T_con(ih,ip) + p_grid_(ip)*(h_T_con(ih,ip) - h_initial) - y_T_1)/(1 + r);
        end
    end
end
clear kappa_cand

figure(455);
subplot(3,1,1);
 plot(a_constr_EGM(:,plot_index_price),a_T_con(:,plot_index_price),'r','Linewidth', 2);
 hold on;
 plot(a_EGM(:,plot_index_price),a_T_unc(:,plot_index_price),'g','Linewidth', 2);
 hold off;
 xlim([-10,20]);
 xlabel('a_{T-1}')
 nice_plot_medium2
 ylabel('a_{T}','Fontsize',15)
subplot(3,1,2);
 plot(a_constr_EGM(:,plot_index_price),c_constr_EGM(:,plot_index_price)/period_years,'r','Linewidth', 2);
 hold on;
 plot(a_EGM(:,plot_index_price),c_EGM(:,plot_index_price)/period_years,'g','Linewidth', 2);
 hold off;
 xlim([-10,20]);
 ylim([0,max(c_EGM(:,plot_index_price)/period_years)]);
 xlabel('a_{T-1}')
 ylabel('c_{T-1} (annual)')
 nice_plot_medium2
subplot(3,1,3);
 plot(a_constr_EGM(:,plot_index_price),h_T_con(:,plot_index_price),'r','Linewidth', 2);
 hold on;
 plot(a_EGM(:,plot_index_price),h_T_unc(:,plot_index_price),'g','Linewidth', 2);
 hold off;
 xlim([-10,20]);
 ylim([0,max(h_T_unc(:,plot_index_price))]);
 xlabel('a_{T-1}')
 ylabel('h_{T}')
 nice_plot_medium2
set(455,'PaperUnits','centimeters','PaperPosition',[0 0 14 16])
% print -r864 -painters -dpng household_T_1.png

if min(min(a_EGM))< min(a_grid_) || min(min(a_constr_EGM))< min(a_grid_)
 disp('Smallest endogenous gridpoint is smaller than smallest gridpoint of a_grid: for excess demand function at smaller a_{T-1}, lower the minimum of a_grid !');
end

c_pol_1 =   NaN*zeros(numb_a_gridpoints,numb_p_gridpoints);
h_pol_1 =   NaN*zeros(numb_a_gridpoints,numb_p_gridpoints);

for ip = 1:numb_p_gridpoints
   sel_none_NaN_constr = ~isnan(a_constr_EGM(:,ip)) & ~isnan(c_constr_EGM(:,ip)) & ~isnan(h_T_con(:,ip));
   sel_none_NaN        = ~isnan(a_EGM(:,ip))        & ~isnan(c_EGM(:,ip))        & ~isnan(h_T_unc(:,ip));
   
   sort_a_c_h_constr = sortrows([a_constr_EGM(sel_none_NaN_constr,ip),c_constr_EGM(sel_none_NaN_constr,ip),...
                                 h_T_con(sel_none_NaN_constr,ip),     a_T_con(sel_none_NaN_constr,ip)],-4);
      
   a_connect = [sort_a_c_h_constr(:,1);a_EGM(sel_none_NaN,ip)];
   c_connect = [sort_a_c_h_constr(:,2);c_EGM(sel_none_NaN,ip)];
   h_connect = [sort_a_c_h_constr(:,3);h_T_unc(sel_none_NaN,ip)];
   
   c_pol_1(:,ip)  = interp1(a_connect,c_connect,a_grid_);
   h_pol_1(:,ip)  = interp1(a_connect,h_connect,a_grid_);
end

excess_demand_1 = h_pol_1 - h_supply;

fig = figure(744);
% % % % % if ~verLessThan('matlab','9.2')
% % % % %    set(fig,'defaultLegendAutoUpdate','off')
% % % % % end
plot_1 = plot(p_grid_,excess_demand_1(plot_index_asset(3),:),'-', 'Linewidth', 1.5);
hold on;
plot_2 = plot(p_grid_,excess_demand_1(plot_index_asset(2),:),'-', 'Linewidth', 1.5);
plot_3 = plot(p_grid_,excess_demand_1(plot_index_asset(1),:),'-', 'Linewidth', 1.5);    
% % % % % legend([plot_1 plot_2 plot_3], ...
% % % % %        ['A_{T-1}= ', num2str(plot_asset_levels_wanted(3))], ...
% % % % %        ['A_{T-1}= ', num2str(plot_asset_levels_wanted(2))], ...
% % % % %        ['A_{T-1}= ', num2str(plot_asset_levels_wanted(1))], 'Location','NorthEast');
plot(p_grid_,zeros(numb_p_gridpoints),'k','Linewidth', 1);
xlabel('p_{T-1}')
ylabel('Excess demand for housing')
hold off;
xlim([-0.05,p_max])
ylim([-1,1.5])
nice_plot_medium2;

set(744,'PaperUnits','centimeters','PaperPosition',[0 0 14 11])
% print -r864 -painters -dpng excess_demand_T_1.png





